"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
const gatsby_theme_aoi_blog_utils_1 = require("@cieloazul310/gatsby-theme-aoi-blog-utils");
function createTerminology(basePath, items) {
    const values = new Set(items);
    return Array.from(values, (name) => ({
        slug: (0, gatsby_theme_aoi_blog_utils_1.createSlug)(basePath, name),
        name,
        totalCount: items.filter((v) => v === name).length,
    })).sort((a, b) => b.totalCount - a.totalCount || a.name.localeCompare(b.name));
}
function mdxPostToMonths(posts, basePaths) {
    const months = posts
        .sort((a, b) => new Date(b.date).getTime() - new Date(a.date).getTime())
        .reduce((accum, { date }) => {
        const year = new Date(date).getFullYear().toString();
        const month = (new Date(date).getMonth() + 1).toString().padStart(2, '0');
        const monthId = `${year}/${month}`;
        const indexInAccum = accum.map((d) => d.id).indexOf(monthId);
        if (indexInAccum < 0) {
            return [
                ...accum,
                {
                    year,
                    month,
                    id: monthId,
                    basePath: `${basePaths.posts}/${year}/${month}`,
                    totalCount: posts.filter((post) => new Date(post.date).getFullYear().toString() === year &&
                        (new Date(post.date).getMonth() + 1)
                            .toString()
                            .padStart(2, '0') === month).length,
                },
            ];
        }
        return accum;
    }, []);
    return months.map(({ year, month, ...props }) => {
        const gte = `${year}-${month}`;
        const lt = month !== '12'
            ? `${year}-${(parseInt(month, 10) + 1).toString().padStart(2, '0')}`
            : `${(parseInt(year, 10) + 1).toString()}-01`;
        return {
            ...props,
            year,
            month,
            gte,
            lt,
        };
    });
}
/**
 * gatsbyCreateResolvers で何をするか
 *
 * 1. 月別の MdxPost を返す Graph クエリを作成
 * 2. 全てのカテゴリー、タグを返すクエリを作成
 */
function gatsbyCreateResolvers({ createResolvers }, themeOptions) {
    const { basePaths } = (0, gatsby_theme_aoi_blog_utils_1.withDefaults)(themeOptions);
    const resolvers = {
        Query: {
            allMdxPostMonths: {
                type: `[MdxPostMonth]!`,
                resolve: async (source, args, context, info) => {
                    const { entries } = await context.nodeModel.findAll({
                        type: `MdxPost`,
                    });
                    return mdxPostToMonths(Array.from(entries), basePaths);
                },
            },
            allCategories: {
                type: `[Terminology]!`,
                resolve: async (source, args, context, info) => {
                    const { entries } = await context.nodeModel.findAll({
                        type: `MdxPost`,
                        query: {
                            filter: {
                                categories: {
                                    ne: null,
                                },
                            },
                        },
                    });
                    const hoge = Array.from(entries).reduce((accum, { categories }) => [...accum, ...(categories ?? [])], []);
                    return createTerminology(basePaths.category, hoge);
                },
            },
            allTags: {
                type: `[Terminology]!`,
                resolve: async (source, args, context, info) => {
                    const { entries } = await context.nodeModel.findAll({
                        type: `MdxPost`,
                        query: {
                            filter: {
                                tags: {
                                    ne: null,
                                },
                            },
                        },
                    });
                    const hoge = Array.from(entries).reduce((accum, { tags }) => [...accum, ...(tags ?? [])], []);
                    return createTerminology(basePaths.tag, hoge);
                },
            },
        },
    };
    createResolvers(resolvers);
}
exports.default = gatsbyCreateResolvers;
