"use strict";
var __createBinding = (this && this.__createBinding) || (Object.create ? (function(o, m, k, k2) {
    if (k2 === undefined) k2 = k;
    var desc = Object.getOwnPropertyDescriptor(m, k);
    if (!desc || ("get" in desc ? !m.__esModule : desc.writable || desc.configurable)) {
      desc = { enumerable: true, get: function() { return m[k]; } };
    }
    Object.defineProperty(o, k2, desc);
}) : (function(o, m, k, k2) {
    if (k2 === undefined) k2 = k;
    o[k2] = m[k];
}));
var __setModuleDefault = (this && this.__setModuleDefault) || (Object.create ? (function(o, v) {
    Object.defineProperty(o, "default", { enumerable: true, value: v });
}) : function(o, v) {
    o["default"] = v;
});
var __importStar = (this && this.__importStar) || function (mod) {
    if (mod && mod.__esModule) return mod;
    var result = {};
    if (mod != null) for (var k in mod) if (k !== "default" && Object.prototype.hasOwnProperty.call(mod, k)) __createBinding(result, mod, k);
    __setModuleDefault(result, mod);
    return result;
};
Object.defineProperty(exports, "__esModule", { value: true });
const path = __importStar(require("path"));
const gatsby_core_utils_1 = require("gatsby-core-utils");
const gatsby_theme_aoi_blog_utils_1 = require("@cieloazul310/gatsby-theme-aoi-blog-utils");
async function processAuthorRelativeImage(source, context, type) {
    const { dir } = source;
    const imagePath = (0, gatsby_core_utils_1.slash)(path.join(dir, (source[type] ?? '')));
    const fileNode = await context.nodeModel.findOne({
        type: `File`,
        query: {
            filter: {
                absolutePath: {
                    eq: imagePath,
                },
            },
        },
    });
    return fileNode ?? undefined;
}
/**
 * createAuthorSchemaCustomization で何をするか
 *
 * `Author` スキーマ(ノード)を拡張する
 * 1. avatar フィールドと `File` ノードを連結する
 * 2. posts フィールドに `MdxPost` ノードを連結する
 */
function createAuthorSchemaCustomization({ actions, schema, }) {
    const { createTypes } = actions;
    createTypes(`
    type AuthorSocial @dontInfer {
      name: String!
      url: String!
    }
    type AuthorMdxPost @dontInfer {
      items: [MdxPost]!
      totalCount: Int!
    }
    type Author implements Node @dontInfer {
      name: String!
      slug: String
      avatar: File
      description: String
      website: String
      websiteURL: String
      socials: [AuthorSocial]
      posts: AuthorMdxPost
    }
  `);
    createTypes(schema.buildObjectType({
        name: `Author`,
        fields: {
            // 1. avatar フィールドと `File` ノードを連結する
            avatar: {
                type: `File`,
                resolve: async (source, args, context, info) => {
                    if (!source.avatar)
                        return null;
                    if (source.image___NODE && (0, gatsby_theme_aoi_blog_utils_1.isString)(source.image___NODE)) {
                        return context.nodeModel.getNodeById({ id: source.image___NODE });
                    }
                    return processAuthorRelativeImage(source, context, `avatar`);
                },
            },
            // 2. posts フィールドに `MdxPost` ノードを連結する
            posts: {
                type: `AuthorMdxPost`,
                resolve: async (source, args, context, info) => {
                    const { entries, totalCount } = await context.nodeModel.findAll({
                        type: `MdxPost`,
                        query: {
                            filter: { author: { name: { eq: source.name } } },
                        },
                    });
                    return {
                        items: entries,
                        totalCount: await totalCount(),
                    };
                },
            },
        },
    }));
}
exports.default = createAuthorSchemaCustomization;
