"use strict";
var __createBinding = (this && this.__createBinding) || (Object.create ? (function(o, m, k, k2) {
    if (k2 === undefined) k2 = k;
    var desc = Object.getOwnPropertyDescriptor(m, k);
    if (!desc || ("get" in desc ? !m.__esModule : desc.writable || desc.configurable)) {
      desc = { enumerable: true, get: function() { return m[k]; } };
    }
    Object.defineProperty(o, k2, desc);
}) : (function(o, m, k, k2) {
    if (k2 === undefined) k2 = k;
    o[k2] = m[k];
}));
var __setModuleDefault = (this && this.__setModuleDefault) || (Object.create ? (function(o, v) {
    Object.defineProperty(o, "default", { enumerable: true, value: v });
}) : function(o, v) {
    o["default"] = v;
});
var __importStar = (this && this.__importStar) || function (mod) {
    if (mod && mod.__esModule) return mod;
    var result = {};
    if (mod != null) for (var k in mod) if (k !== "default" && Object.prototype.hasOwnProperty.call(mod, k)) __createBinding(result, mod, k);
    __setModuleDefault(result, mod);
    return result;
};
Object.defineProperty(exports, "__esModule", { value: true });
const path = __importStar(require("path"));
const gatsby_core_utils_1 = require("gatsby-core-utils");
const gatsby_theme_aoi_blog_utils_1 = require("@cieloazul310/gatsby-theme-aoi-blog-utils");
function mdxResolverPassthrough(fieldName) {
    return async (source, args, context, info) => {
        const type = info.schema.getType(`Mdx`);
        const mdxNode = context.nodeModel.getNodeById({
            id: source.parent,
        });
        const resolver = type?.getFields()[fieldName].resolve;
        if (!resolver)
            return {};
        if (!mdxNode)
            return {};
        const result = await resolver(mdxNode, args, context, {
            fieldName,
        });
        return result;
    };
}
async function processMdxPostRelativeImage(source, context, type) {
    // Image is a relative path - find a corresponding file
    const mdxFileNode = context.nodeModel.findRootNodeAncestor(source, (node) => node.internal && node.internal.type === `File`);
    if (!mdxFileNode) {
        return undefined;
    }
    const imagePath = (0, gatsby_core_utils_1.slash)(path.join(mdxFileNode.dir, (source[type] ?? '')));
    const fileNode = await context.nodeModel.findOne({
        type: `File`,
        query: {
            filter: {
                absolutePath: {
                    eq: imagePath,
                },
            },
        },
    });
    return fileNode ?? undefined;
}
/**
 * createMdxPostSchemaCustomization で何をするか
 *
 * `MdxPost` スキーマ(ノード)を拡張する
 * 1. author フィールドと `Author` ノードを連結する
 * 2. image フィールドと `File` ノードを連結する
 * 3. tableOfContents, excerpt フィールドを `Mdx` ノードから引用する
 *
 * その他のスキーマを作成する
 * 1. Terminology スキーマ (カテゴリー、タグで使用)
 * 2. MdxPostMonth スキーマ (月別の MdxPost で使用)
 */
function createMdxPostSchemaCustomization({ actions, schema }, themeOptions) {
    const options = (0, gatsby_theme_aoi_blog_utils_1.withDefaults)(themeOptions);
    const { createTypes } = actions;
    createTypes(`
    type Terminology @dontInfer {
      name: String!
      slug: String!
      totalCount: Int!
    }
    type MdxPostMonth {
      id: String!
      year: String!
      month: String!
      basePath: String!
      gte: String!
      lt: String!
      totalCount: Int!
    }
  `);
    createTypes(schema.buildObjectType({
        name: `MdxPost`,
        fields: {
            id: { type: `ID!` },
            title: { type: `String!` },
            slug: { type: `String!` },
            date: { type: `Date!`, extensions: { dateformat: {} } },
            // 1. author フィールドと `Author` ノードを連結する
            author: {
                type: `Author!`,
                resolve: async (source, args, context, info) => {
                    const author = await context.nodeModel.findOne({
                        type: 'Author',
                        query: {
                            filter: { name: { eq: source.author } },
                        },
                    });
                    return (author ?? {
                        name: source.author ?? 'Unknown author',
                    });
                },
            },
            categories: {
                type: `[String]!`,
            },
            tags: {
                type: `[String]!`,
            },
            // 2. image フィールドと `File` ノードを連結する
            image: {
                type: `File`,
                resolve: async (source, args, context, info) => {
                    if (source.image___NODE && (0, gatsby_theme_aoi_blog_utils_1.isString)(source.image___NODE)) {
                        return context.nodeModel.getNodeById({ id: source.image___NODE });
                    }
                    return processMdxPostRelativeImage(source, context, `image`);
                },
            },
            imageAlt: { type: `String` },
            // 3. tableOfContents, excerpt フィールドを `Mdx` ノードから引用する
            tableOfContents: {
                type: `JSON`,
                args: {
                    maxDepth: {
                        type: `Int`,
                    },
                },
                resolve: mdxResolverPassthrough(`tableOfContents`),
            },
            excerpt: {
                type: `String!`,
                args: {
                    pruneLength: {
                        type: `Int`,
                        defaultValue: options.excerptLength,
                    },
                },
                resolve: mdxResolverPassthrough(`excerpt`),
            },
        },
        interfaces: [`Node`],
        extensions: {
            infer: false,
        },
    }));
}
exports.default = createMdxPostSchemaCustomization;
