"use strict";
var __createBinding = (this && this.__createBinding) || (Object.create ? (function(o, m, k, k2) {
    if (k2 === undefined) k2 = k;
    var desc = Object.getOwnPropertyDescriptor(m, k);
    if (!desc || ("get" in desc ? !m.__esModule : desc.writable || desc.configurable)) {
      desc = { enumerable: true, get: function() { return m[k]; } };
    }
    Object.defineProperty(o, k2, desc);
}) : (function(o, m, k, k2) {
    if (k2 === undefined) k2 = k;
    o[k2] = m[k];
}));
var __setModuleDefault = (this && this.__setModuleDefault) || (Object.create ? (function(o, v) {
    Object.defineProperty(o, "default", { enumerable: true, value: v });
}) : function(o, v) {
    o["default"] = v;
});
var __importStar = (this && this.__importStar) || function (mod) {
    if (mod && mod.__esModule) return mod;
    var result = {};
    if (mod != null) for (var k in mod) if (k !== "default" && Object.prototype.hasOwnProperty.call(mod, k)) __createBinding(result, mod, k);
    __setModuleDefault(result, mod);
    return result;
};
Object.defineProperty(exports, "__esModule", { value: true });
const path = __importStar(require("path"));
const gatsby_core_utils_1 = require("gatsby-core-utils");
const gatsby_source_filesystem_1 = require("gatsby-source-filesystem");
const gatsby_theme_aoi_blog_utils_1 = require("@cieloazul310/gatsby-theme-aoi-blog-utils");
function isMdxNode(node) {
    return typeof node.frontmatter === 'object';
}
/**
 * onCreateNode で何をするか
 *
 * 1. gatsby-plugin-mdx が生成した `Mdx` ノードから新たに `MdxPost` ノードを生成する
 * 2. frontmatter.image の URL から remoteFileNode を生成する
 */
async function onCreateNode({ node, actions: { createNode, createParentChildLink }, getNode, createNodeId, getCache, }, themeOptions) {
    const options = (0, gatsby_theme_aoi_blog_utils_1.withDefaults)(themeOptions);
    // you only want to operate on `Mdx` nodes. If you had content from a
    // remote CMS you could also check to see if the parent node was a
    // `File` node here
    if (node.internal.type !== 'Mdx')
        return;
    if (!isMdxNode(node))
        return;
    const contentPath = path.join(options.contentPath, options.basePaths.posts);
    // Create source field (according to contentPath)
    const parentFileNode = getNode(node.parent ?? '');
    const source = parentFileNode?.sourceInstanceName;
    // pages/*.mdx を除外
    if (source !== contentPath)
        return;
    const value = (0, gatsby_source_filesystem_1.createFilePath)({ node, getNode });
    const slug = path.join(options.basePaths.posts, value);
    const fieldData = {
        title: node.frontmatter.title,
        slug,
        date: node.frontmatter.date,
        author: node.frontmatter.author,
        categories: node.frontmatter.categories ?? [],
        tags: node.frontmatter.tags ?? [],
        image: node.frontmatter.image,
        imageAlt: node.frontmatter.imageAlt,
    };
    // 2. frontmatter.image の URL から remoteFileNode を生成する
    const { image } = node.frontmatter;
    if (image) {
        if ((0, gatsby_theme_aoi_blog_utils_1.validURL)(image)) {
            const remoteFileNode = await (0, gatsby_source_filesystem_1.createRemoteFileNode)({
                url: image,
                parentNodeId: node.id,
                createNode,
                createNodeId,
                getCache,
            });
            if (remoteFileNode) {
                fieldData.image___NODE = remoteFileNode.id;
            }
        }
    }
    // 1. gatsby-plugin-mdx が生成した `Mdx` ノードから新たに `MdxPost` ノードを生成する
    const mdxPostId = createNodeId(`${node.id} >>> MdxPost`);
    await createNode({
        ...fieldData,
        // Required fields.
        id: mdxPostId,
        parent: node.id,
        children: [],
        internal: {
            type: `MdxPost`,
            contentDigest: (0, gatsby_core_utils_1.createContentDigest)(fieldData),
            content: JSON.stringify(fieldData),
            description: `Mdx implementation of the BlogPost interface`,
            contentFilePath: node.internal.contentFilePath,
        },
    });
    const mdxPostNode = getNode(mdxPostId);
    if (mdxPostNode) {
        createParentChildLink({ parent: node, child: mdxPostNode });
    }
}
exports.default = onCreateNode;
