"use strict";
var __createBinding = (this && this.__createBinding) || (Object.create ? (function(o, m, k, k2) {
    if (k2 === undefined) k2 = k;
    var desc = Object.getOwnPropertyDescriptor(m, k);
    if (!desc || ("get" in desc ? !m.__esModule : desc.writable || desc.configurable)) {
      desc = { enumerable: true, get: function() { return m[k]; } };
    }
    Object.defineProperty(o, k2, desc);
}) : (function(o, m, k, k2) {
    if (k2 === undefined) k2 = k;
    o[k2] = m[k];
}));
var __setModuleDefault = (this && this.__setModuleDefault) || (Object.create ? (function(o, v) {
    Object.defineProperty(o, "default", { enumerable: true, value: v });
}) : function(o, v) {
    o["default"] = v;
});
var __importStar = (this && this.__importStar) || function (mod) {
    if (mod && mod.__esModule) return mod;
    var result = {};
    if (mod != null) for (var k in mod) if (k !== "default" && Object.prototype.hasOwnProperty.call(mod, k)) __createBinding(result, mod, k);
    __setModuleDefault(result, mod);
    return result;
};
Object.defineProperty(exports, "__esModule", { value: true });
const fs = __importStar(require("fs"));
const path = __importStar(require("path"));
const yaml = __importStar(require("yaml"));
const gatsby_source_filesystem_1 = require("gatsby-source-filesystem");
const gatsby_theme_aoi_blog_utils_1 = require("@cieloazul310/gatsby-theme-aoi-blog-utils");
/**
 * sourceNodes で何をするか
 *
 * 1. プロジェクト内の authors.yml から `Author` ノードを生成
 * 2. Author の avatar がリモートファイルの場合 remoteFileNode を生成する
 */
async function sourceNodes({ actions, createNodeId, createContentDigest, getCache }, themeOptions) {
    const { basePaths, contentPath } = (0, gatsby_theme_aoi_blog_utils_1.withDefaults)(themeOptions);
    const { createNode } = actions;
    const authorsFile = path.resolve(contentPath, 'authors.yml');
    const dir = path.dirname(authorsFile);
    if (!fs.existsSync(authorsFile))
        return;
    const authors = yaml.parse(fs.readFileSync(authorsFile, 'utf8'));
    await Promise.all(authors.map(async (author) => {
        const data = {
            ...author,
            dir,
            slug: (0, gatsby_theme_aoi_blog_utils_1.createSlug)(basePaths.author, author.name),
        };
        const nodeId = createNodeId(`author-${author.name}`);
        const { avatar } = author;
        if (avatar) {
            if ((0, gatsby_theme_aoi_blog_utils_1.validURL)(avatar)) {
                const remoteFileNode = await (0, gatsby_source_filesystem_1.createRemoteFileNode)({
                    url: avatar,
                    parentNodeId: nodeId,
                    createNode,
                    createNodeId,
                    getCache,
                });
                if (remoteFileNode) {
                    data.image___NODE = remoteFileNode.id;
                }
            }
        }
        const nodeContent = JSON.stringify(data);
        const nodeMeta = {
            id: nodeId,
            parent: null,
            children: [],
            internal: {
                type: `Author`,
                mediaType: `text/html`,
                content: nodeContent,
                contentDigest: createContentDigest(data),
            },
        };
        const node = { ...data, ...nodeMeta };
        createNode(node);
    }));
}
exports.default = sourceNodes;
